@@ -23,6 +23,36 @@
 #include <qpainter.h>
 #include <qpixmap.h>
 #include <qfont.h>
+#include <qstylesheet.h>
+
+#if QT_VERSION >= 300
+
+#include <qregexp.h>
+
+bool CMakeOutputWidget::isAboveQt303()
+{
+  static int result = -1;
+
+  if ( result == -1 ) {
+    int qMajor = -1;
+    int qMinor = -1;
+    int qVer = -1;
+    QRegExp verRegExp("(\\d+)\\.(\\d+)\\.(\\d+)");
+    verRegExp.search( QString(qVersion()) );
+    qVer = verRegExp.cap(1).toInt();
+    qMajor = verRegExp.cap(2).toInt();
+    qMinor = verRegExp.cap(3).toInt();
+    //qDebug( "Qt version: %d.%d.%d", qVer, qMajor, qMinor );
+    if ( qVer > 3 || qMajor > 0 || qMinor > 3 ) {
+      result = 1;
+    } else {
+      result = 0;
+    }
+  }
+
+  return (result > 0);
+}
+#endif
 
 COutputWidget::COutputWidget(QWidget* parent, const char* name) :
 //  KEdit(parent,name)
@@ -73,20 +103,21 @@ void COutputWidget::insertAtEnd(const QString& s)
 CMakeOutputWidget::CMakeOutputWidget(QWidget* parent, const char* name) :
   QMultiLineEdit(parent, name),
   m_buf(),
-  m_dirChange("[^\n]*: ([^\n]+) (`|)(.*)('|)([^\n]*)\n"),
+  m_dirChange("[^\n]*: ([^\n]+) (`|)(.*)('|)([^\n]*)(\n|)"),
   m_errorGcc("^([^: \t]+):([0-9]+)[:,].*")
 {
   it = m_errorMap.begin();
   setReadOnly(true);        // -JROC uncommented again
-#if QT_VERSION >= 300
-  setTextFormat(Qt::RichText);
-  setWordWrap(NoWrap);
-
-  // This is necessary in order to work around a bug in Qt's RichText widget. (Marcus Gruendler)
-  styleSheet()->item( "font" )->setDisplayMode( QStyleSheetItem::DisplayBlock );
-#else
   setWordWrap(WidgetWidth); // -JROC
   setWrapPolicy(Anywhere); // -JROC
+#if QT_VERSION >= 300
+  setTextFormat(Qt::RichText);
+
+  if (!isAboveQt303()) {
+    // This is necessary in order to work around a bug in Qt's RichText widget. (Marcus Gruendler)
+    styleSheet()->item( "font" )->setDisplayMode( QStyleSheetItem::DisplayBlock );
+  }
+	connect(this, SIGNAL(clicked(int,int)), SLOT(checkForError(int,int)));
 #endif
 }
 
@@ -105,12 +136,16 @@ void CMakeOutputWidget::insertAtEnd(const QString& text, MakeOutputErrorType def
     // extract file info from this line - if any
     processLine(line, defaultType);
 
+    bool displayAdditions=atEnd();
 #if QT_VERSION < 300
     // add to the end of the text - highlighting is done in the
     // paint routine.
     int row = (numLines() < 1)? 0 : numLines()-1;
     int col = qstrlen(textLine(row));
-    bool displayAdditions=atEnd();
+    
+    // escapes the string 
+    // line = QStyleSheet::convertFromPlainText( line );
+
     insertAt(line, row, col);
     if (displayAdditions)
       setCursorPosition(numLines()+1,0);
@@ -119,8 +154,17 @@ void CMakeOutputWidget::insertAtEnd(const QString& text, MakeOutputErrorType def
     int currentPara;
     int index;
     getCursorPosition(&currentPara, &index);
-    int paraCount = paragraphs()-1;
-    bool displayAdditions = (paraCount == currentPara);
+
+    int paraCount;
+    if (isAboveQt303()) {
+      paraCount = paragraphs();
+    }
+    else {
+      paraCount = paragraphs()-1;
+    }    
+
+    // escape rich edit tags like "&", "<", ">"
+    line = QStyleSheet::escape( line );
 
     switch (lineType(paraCount))
     {
@@ -133,15 +177,21 @@ void CMakeOutputWidget::insertAtEnd(const QString& text, MakeOutputErrorType def
         append(line);
         break;
       default:
-        append("<font color=\"black\">" + line + "</font>");
+        // Runtime check for broken Qt version. Don't add empty new lines.
+        if (line[0]=='\n' && (isAboveQt303())) {
+          break;
+        }
+          
+        line ="<font color=\"black\">" + line + "</font>";
+        append(line);
         break;
     }
 
-    if (displayAdditions)
-      setCursorPosition(paraCount+1, 0);
-
+    if (displayAdditions) {
+      moveCursor(QTextEdit::MoveEnd, false);
+    }
     ensureCursorVisible();
-
+    
 #endif
   }
 }
@@ -215,9 +265,10 @@ void CMakeOutputWidget::processLine(const QString& line, MakeOutputErrorType typ
     kdDebug() << "behind:" << msgBehind << endl << dbg << endl;
 */
 
-    // these are the strings that gmake 3.79.1 outputs on entering a directory
+    // these are the strings that gmake 3.79.1 (or later versions of it) outputs on entering a directory
     if ( msg == "Entering directory" || 			// English - default
          msg == "Wechsel in das Verzeichnis Verzeichnis" ||    // German - yes, there is a spelling error in make :)
+         msg == "Wechsel in das Verzeichnis" ||    // German - gmake without that spelling error
          msg == "Cambiando a directorio" || // Spanish
          msg == fr_e || // French
          msg == ja_e || // Japanese
@@ -246,6 +297,7 @@ void CMakeOutputWidget::processLine(const QString& line, MakeOutputErrorType typ
 
     if ( msg == "Leaving directory" || // en
          msg == "Verlassen des Verzeichnisses Verzeichnis" || // de
+         msg == "Verlassen des Verzeichnisses" ||    // de - gmake without that spelling error
          msg == "Saliendo directorio" || // es
          msg == fr_l || // fr
          msg == ja_l || //ja
@@ -275,7 +327,16 @@ void CMakeOutputWidget::processLine(const QString& line, MakeOutputErrorType typ
   {
     // add the error keyed on the line number in the make output widget
     ErrorDetails errorDetails(fileInErr, lineInErr, type);
-    m_errorMap.insert(numLines()-1, errorDetails);
+#if QT_VERSION >= 300
+    if (isAboveQt303()) {
+      m_errorMap.insert(numLines(), errorDetails);
+    }
+    else {
+      m_errorMap.insert(numLines()-1, errorDetails);
+    }
+#else
+      m_errorMap.insert(numLines()-1, errorDetails);
+#endif
   }
 }
 
@@ -333,7 +394,7 @@ void CMakeOutputWidget::selectLine(int line)
 }
 
 // --------------------------------------------------------------------------------
-
+#if QT_VERSION < 300
 // override from QMultiLineEdit
 void CMakeOutputWidget::keyPressEvent( QKeyEvent* event )
 {
@@ -351,20 +412,25 @@ void CMakeOutputWidget::keyPressEvent( QKeyEvent* event )
 }
 
 // --------------------------------------------------------------------------------
-
 void CMakeOutputWidget::mouseReleaseEvent(QMouseEvent* event)
 {
   QMultiLineEdit::mouseReleaseEvent(event);
   checkForError();
 }
+#endif
 
 // --------------------------------------------------------------------------------
 
 // Check the current line for errors _and_ do the switch to the file:linenumber.
+#if QT_VERSION < 300
 void CMakeOutputWidget::checkForError()
 {
   int line,col;
   getCursorPosition(&line,&col);
+#else
+void CMakeOutputWidget::checkForError(int line, int col)
+{
+#endif
 
   ErrorMap::Iterator tmp_it;
   if ((tmp_it = m_errorMap.find(line)) != m_errorMap.end())
