@@ -29,7 +29,7 @@
 #include "khtmlview.h"
 #include "qextmdimainfrm.h"
 
-#include <iostream.h>
+#include <iostream>
 using namespace std;
 
 
@@ -317,6 +317,7 @@ void KDevSession::recreateDocs(QDomElement& el){
   if (bTaskBarWasOn) {
      pMainWidget->showViewTaskBar();
   }
+  delete winListIter; winListIter = NULL;
 }
 
 //-------------- Falk's Views recreation ---------//
