@@ -21,6 +21,7 @@
 #include <qdir.h>
 #include <qprinter.h>
 #include <qprintdialog.h>
+#include <kprinter.h>
 #include <qpaintdevicemetrics.h>
 #include <qclipbrd.h>
 
@@ -57,7 +58,7 @@ QPixmap *getBuffer(void *user) {
   BufferInfo *info;
 
   if (!buffer) buffer = new QPixmap;
-  info = new BufferInfo;
+  info = new BufferInfo;   // causes memory leak but don't know how to handle (Zu)
   info->user = user;
   info->w = 0;
   info->h = 0;
@@ -93,7 +94,11 @@ void releaseBuffer(void *user) {
 
   for (z = (int) bufferInfoList.count() -1; z >= 0 ; z--) {
     info = bufferInfoList.at(z);
-    if (info->user == user) bufferInfoList.remove(z);
+    if (info->user == user) 
+    {
+      bufferInfoList.remove(z);
+      delete info;
+    }
   }
   resizeBuffer(0,0,0);
 }
@@ -1200,10 +1205,11 @@ X      : cut
               else kWrite->toggleOverwrite();
             break;
         case Key_Tab:
-            //if (e->state() & ShiftButton) ; // TODO: jump to enclosing brace
-              //else
             kWriteDoc->tab(this,c);
             break;
+        case Key_BackTab:
+            kWriteDoc->shiftTab(this,c);
+            break;
 /*        case Key_F9:
             printf("text() %s\n", kWrite->text().data());
             break;
@@ -2422,7 +2428,7 @@ void KWrite::readConfig(KConfig *config) {
 
   searchFlags = config->readNumEntry("SearchFlags",sfPrompt);
   configFlags = config->readNumEntry("ConfigFlags",
-    cfPersistent | cfRemoveSpaces) & ~cfMark;
+    cfPersistent | cfRemoveSpaces | cfAutoIndent | cfIndentBraces | cfBackspaceIndent) & ~cfMark;
   wrapAt = config->readNumEntry("WrapAt",78);
 }
 
@@ -2510,23 +2516,28 @@ void KWrite::print() {
   int fontHeight;
   int lines, pages;
   int line, c, p, l, y;
-  QPrinter prt;
+  KPrinter prt;
+
+  prt.setFullPage(false);
 
   if (prt.setup()) {
-    QPaintDeviceMetrics pdm(&prt);
     QPainter paint;
-
-    paint.begin(&prt);
-    fontHeight = kWriteDoc->fontHeight;
+    QPaintDeviceMetrics pdm(&prt);
+    fontHeight = kWriteDoc->printFontHeight;
     lines = pdm.height()/fontHeight;
     pages = (kWriteDoc->lastLine() +1)/lines;
+
+    paint.begin(&prt);
     for (c = 1; c <= prt.numCopies(); c++) {
       line = 0;
-      for (p = 0; p <= pages; p++) {
+      
+      for (p =  0 ; p <= pages; p++) {
         y = 0;
-        if (prt.pageOrder() == QPrinter::LastPageFirst) line = (pages - p)*lines;
+        if (prt.pageOrder() == KPrinter::LastPageFirst) 
+          line = (pages - p)*lines;
+        
         for (l = 0; l < lines; l++) {
-          kWriteDoc->printTextLine(paint, line, pdm.width(), y);
+          kWriteDoc->paintTextLine(paint, line, y, 0, pdm.width(), flags() & cfShowTabs, true);
           line++;
           y += fontHeight;
         }
